/**
 * @file main.c
 * @author Nippon Pulse Motor Co., Ltd.
 * @brief メインソース
 * @version 0.1
 * @date 2023-06-07
 * 
 * @copyright Copyright (c) 2023
 * 
 */

#include "stdafx.h"
#include "main.h"
#include "pcl_access.h"

/**
 * @brief メイン
 * 
 */
void main( void ) {

    const PIO pio = pio0;                                                                                       /* PIO0を使用する */
    const uint sm = 1;                                                                                          /* PCL6046パラレスアクセス用ステートマシン使用番号 */

    stdio_init_all();                                                                                           /* To use USB */
    sleep_ms( 3000 );		                                                                                    /* wait serial connection */
    
    pcl_bus_init( pio, sm, IFB, PIO_OUT_BASE, PIO_OUT_NUM, PIO_SIDE_BASE, PIO_SIDE_BASE_NUM, PIO_IN_BASE );     /* PCL6046 パラレルバス初期化 */
    
    pcl_reg_set( pio, sm );                                                                                     /* PCL6046 各種レジスタ設定(X軸,Y軸) */
    
    org_run( pio, sm );                                                                                         /* X軸,Y軸原点位置移動 */
    
	arc_init_pos( pio, sm );                                                                                    /* 円弧補間初期位置へ移動 */
    sleep_ms( 1000 );
	arc_run( pio, sm );                                                                                         /* 円弧補間スタート */

}

/**
 * @brief 円弧補間スタート
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 */
void arc_run( PIO pio, uint sm ){

    const uint32_t com_axis = 0x03;                                      /* コマンド書き込み時軸選択(X軸,Y軸選択) */
	
	printf( "%s\n", "PRMD" );                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, X_SEL, PRMD, 0x00000064 );              /* 動作モード=CW方向円弧補間(X軸) */
    read_reg_pcl( pio, sm, IFB, X_SEL, PRMD );                           /* PRMDレジスタ読み出し(X軸) */
	
	printf( "%s\n", "PRMD" );                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, Y_SEL, PRMD, 0x00000064 );              /* 動作モード=CW方向円弧補間(Y軸) */
    read_reg_pcl( pio, sm, IFB, Y_SEL, PRMD );                           /* PRMDレジスタ読み出し(Y軸) */
	
	printf( "%s\n", "PRMV" );                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, X_SEL, PRMV, 0x00000000 );              /* PRMV=0(X軸) */
    read_reg_pcl( pio, sm, IFB, X_SEL, PRMV );                           /* PRMVレジスタ読み出し(X軸) */
	
	printf( "%s\n", "PRMV" );                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, Y_SEL, PRMV, 0x00000000 );              /* PRMV=0(Y軸) */
    read_reg_pcl( pio, sm, IFB, Y_SEL, PRMV );                           /* PRMVレジスタ読み出し(Y軸) */
	
	write_com_pcl( pio, sm, IFB, com_axis, COMB, STAUD );                /* STAUDで動作スタート(X軸,Y軸) */

}

/**
 * @brief 円弧補間初期位置
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 */
void arc_init_pos( PIO pio, uint sm ){

    const uint32_t com_axis = 0x03;                                      /* コマンド書き込み時軸選択(X軸,Y軸選択) */
    
    printf( "%s\n", "PRMD" );                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, X_SEL, PRMD, 0x00000041 );              /* 動作モード=位置決め相対移動モードに設定(X軸) */
    read_reg_pcl( pio, sm, IFB, X_SEL, PRMD );                           /* PRMDレジスタ読み出し(X軸) */

    printf( "%s\n", "PRMD" );                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, Y_SEL, PRMD, 0x00000041 );              /* 動作モード=位置決め相対移動モードに設定(Y軸) */
    read_reg_pcl( pio, sm, IFB, Y_SEL, PRMD );                           /* PRMDレジスタ読み出し(Y軸) */
	
    printf( "%s\n", "PRMV" );                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, X_SEL, PRMV, 0x00000032 );              /* PRMV=50(X軸) */
    read_reg_pcl( pio, sm, IFB, X_SEL, PRMV );                           /* PRMVレジスタ読み出し(X軸) */

	printf( "%s\n", "PRMV" );                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, Y_SEL, PRMV, 0x000000FA );              /* PRMV=250(Y軸) */
    read_reg_pcl( pio, sm, IFB, Y_SEL, PRMV );                           /* PRMVレジスタ読み出し(Y軸) */
    
    write_com_pcl( pio, sm, IFB, com_axis, COMB, STAUD );                /* STAUDで動作スタート(X軸,Y軸) */

    msts_chk( pio, sm );                                                 /* 動作が停止(BSY=H)になるまで待機 */

    write_com_pcl( pio, sm, IFB, com_axis, COMB, CUN1R );                /* カウンター1のカウント値(指令パルスカウンタ)をクリア(X軸,Y軸) */
}

/**
 * @brief メインステータスBSY確認
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 */
void msts_chk( PIO pio, uint sm ){

    uint32_t msts_x_axis = 0;                                            /* メインステータス(X軸) */
    uint32_t msts_y_axis = 0;                                            /* メインステータス(Y軸) */
    uint32_t busy_x = 0;                                                 /* BSYステータス(X軸) */
    uint32_t busy_y = 0;                                                 /* BSYステータス(Y軸) */

    do{
        printf("MSTS_X\n");
        msts_x_axis = read_msts_pcl(pio, sm, IFB, X_SEL, MSTS);          /* メインステータス読み出し(X軸) */
        msts_y_axis = read_msts_pcl(pio, sm, IFB, Y_SEL, MSTS);          /* メインステータス読み出し(Y軸) */

        busy_x = msts_x_axis & 0x0002;                                   /* BSYステータスフラグ(X軸) */
        busy_y = msts_y_axis & 0x0002;                                   /* BSYステータスフラグ(Y軸) */

    }while (busy_x != 0 | busy_y != 0);                                  /* X軸かY軸どちらか動作中の場合はループする */
}

/**
 * @brief センサー原点位置移動
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 */
void org_run( PIO pio, uint sm ){

    const uint32_t com_axis = 0x03;                                      /* コマンド書き込み時軸選択(X軸,Y軸選択) */

    write_com_pcl( pio, sm, IFB, com_axis, COMB, STAUD );                /* STAUDで動作スタート(X軸,Y軸) */
}

/**
 * @brief PCL初期設定レジスタ
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 */
void pcl_reg_set( PIO pio, uint sm ){

    const uint32_t reg_write_data_x_axis[] = { 0x00000032, 0x00000190, 0x000005AA, 0x00000000, 0x0000012B,     /* PRFL, PRFH, PRUR, PRDR,  PRMG   PRFL=50PPS, PRFH=400PPS, PRMG=1倍*/
                                               0x00000000, 0x00000000, 0x00000064, 0x00000064, 0x00000096,     /* PRUS, PRDS, RFA,  PRMV,  PRIP(半径=150) */
                                               0x00000000, 0x00000000, 0x0000001D, 0x80000000, 0x00100000 };   /* PRCI, PRDP, PRMD(-方向へ原点サーチ), RENV1, RENV3 */
	
	const uint32_t reg_write_data_y_axis[] = { 0x00000032, 0x00000190, 0x000005AA, 0x00000000, 0x0000012B,     /* PRFL, PRFH, PRUR, PRDR,  PRMG   PRFL=50PPS, PRFH=400PPS, PRMG=1倍*/
                                               0x00000000, 0x00000000, 0x00000064, 0x00000064, 0x00000000,     /* PRUS, PRDS, RFA,  PRMV,  PRIP */
                                               0x00000000, 0x00000000, 0x0000001D, 0x80000000, 0x00100000 };   /* PRCI, PRDP, PRMD(-方向へ原点サーチ), RENV1, RENV3 */
    
    const uint32_t reg_address[] = { PRFL, PRFH, PRUR, PRDR,  PRMG,
                                     PRUS, PRDS, RFA,  PRMV,  PRIP,
                                     PRCI, PRDP, PRMD, RENV1, RENV3 };

    const char *reg_name[] = { "PRFL", "PRFH", "PRUR", "PRDR",  "PRMG",
                               "PRUS", "PRDS", "RFA",  "PRMV",  "PRIP",
                               "PRCI", "PRDP", "PRMD", "RENV1", "RENV3" };

    for( int i = 0; i < 15; i++ ){
        printf( "%s\n", reg_name[i] );                                                      /* UARTで送信 */
        write_reg_pcl( pio, sm, IFB, X_SEL, reg_address[i], reg_write_data_x_axis[i] );     /* PCL6046レジスタ書き込み(X軸) */
        read_reg_pcl( pio, sm, IFB, X_SEL, reg_address[i] );                                /* PCL6046レジスタ読み出し(X軸) */
		
		printf( "%s\n", reg_name[i] );                                                      /* UARTで送信 */
        write_reg_pcl( pio, sm, IFB, Y_SEL, reg_address[i], reg_write_data_y_axis[i] );     /* PCL6046レジスタ書き込み(Y軸) */
        read_reg_pcl( pio, sm, IFB, Y_SEL, reg_address[i] );                                /* PCL6046レジスタ書き込み(Y軸) */
    }

    printf( "MSTS_X\n" );
    read_msts_pcl( pio, sm, IFB, X_SEL, MSTS );                                             /* メインステータス読み出し(X軸) */
	
	printf( "MSTS_Y\n" );
    read_msts_pcl( pio, sm, IFB, Y_SEL, MSTS );                                             /* メインステータス読み出し(Y軸) */

}