/**
 * @file pcl_access.c
 * @author Nippon Pulse Motor Co., Ltd.
 * @brief PCL6046アクセス用(Z80フルアドレスでアクセス)
 * @version 0.1
 * @date 2023-06-07
 * 
 * @copyright Copyright (c) 2023
 * 
 */

#include "stdafx.h"
#include "pcl_access.h"
#include "parallel_bus_module.pio.h"                                    /* パラレルバスモジュール */

/**
 * @brief PCLバスライン入出力方向切り替え
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param dir バス入出力方向(true = 出力,false = 入力)
 */
void pcl_bus_dir_change( PIO pio, uint sm, bool dir ){
    pio_sm_set_enabled( pio, sm, false );                               /* PIOステートマシンを無効化 */
    pio_sm_set_consecutive_pindirs( pio, sm, 10, 8, dir );              /* true = PCLデータバスの方向を出力に設定 */
                                                                        /* false = PCLデータバスの方向を入力に設定 */
    pio_sm_set_enabled( pio, sm, true );                                /* PIOステートマシンを有効化 */
}

/**
 * @brief PCLアドレスデータ分割
 * 
 * @param address PCLアドレス値
 * @param pcl_address_bytes PCLアドレスデータ(バイトごとに分割)
 */
void address_byte( uint32_t address, uint32_t pcl_address_bytes[] ){
    pcl_address pcl_address;

    pcl_address.address = address;
    pcl_address_bytes[0] = pcl_address.address_bytes.address_bytes_4;     /* PCLアドレスの最下位アドレス */
    pcl_address_bytes[1] = pcl_address.address_bytes.address_bytes_3;     /* PCLアドレスの下位アドレス */
    pcl_address_bytes[2] = pcl_address.address_bytes.address_bytes_2;     /* PCLアドレスの上位アドレス */
    pcl_address_bytes[3] = pcl_address.address_bytes.address_bytes_1;     /* PCLアドレスの最上位アドレス */
}

/**
 * @brief レジスタ書き込みデータ分割
 * 
 * @param data レジスタ書き込み値
 * @param reg_data_bytes レジスタ書き込みデータ(バイトごとに分割)
 */
void data_byte( uint32_t data, uint32_t reg_data_bytes[] ){
    write_data write_data;

    write_data.data = data;
    reg_data_bytes[0] = write_data.data_bytes.data_bytes_1;               /* PCLレジスタ書き込みの最下位データ */
    reg_data_bytes[1] = write_data.data_bytes.data_bytes_2;               /* PCLレジスタ書き込みの下位データ */
    reg_data_bytes[2] = write_data.data_bytes.data_bytes_3;               /* PCLレジスタ書き込みの上位データ */
    reg_data_bytes[3] = write_data.data_bytes.data_bytes_4;               /* PCLレジスタ書き込みの最上位データ */
}

/**
 * @brief PIOステートマシンのTX_FIFOに書き込み
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param write_data TX_FIFO書き込みデータ
 */
void write_tx_fifo( PIO pio, uint sm, uint32_t write_data ){
    while( pio_sm_is_tx_fifo_empty(pio,sm) == false );                    /* PIOのTX_FIFOが空でないなら無限待機 */
    pio_sm_put_blocking( pio, sm, 1 );                                    /* PCL書き込みモードに設定 */
    while( pio_sm_is_tx_fifo_empty(pio,sm) == false );                    /* PIOのTX_FIFOが空でないなら無限待機 */
    pio_sm_put_blocking( pio, sm, write_data );                           /* PCLレジスタ書き込みデータ+軸選択データ+PCLアドレスデータをTX_FIFOに書き込み
                                                                            (すなわちGPIOからデータが出力される) */
}

/**
 * @brief PIOステートマシンのRX_FIFOからデータ読み出し
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param write_data TX_FIFO書き込みデータ
 * @return uint32_t RX_FIFO読み出しデータ
 */
uint32_t read_rx_fifo( PIO pio, uint sm, uint32_t write_data ){
    uint32_t read_data_buf = 0;

    while( pio_sm_is_tx_fifo_empty(pio,sm) == false );                    /* PIOのTX_FIFOが空でないなら無限待機 */
    pio_sm_put_blocking( pio, sm, 0 );                                    /* PCL読み出しモードに設定 */
    while( pio_sm_is_tx_fifo_empty(pio,sm) == false );                    /* PIOのTX_FIFOが空でないなら無限待機 */
    pio_sm_put_blocking( pio, sm, write_data );                           /* PCLレジスタ書き込みデータ+軸選択データ+PCLアドレスデータをTX_FIFOに書き込み
                                                                            (すなわちGPIOからデータが出力される) */
    while( pio_sm_is_rx_fifo_empty(pio,sm) == true );                     /* PIOのRX_FIFOが空（読み出しデータが入っていない)時は無限待機 */
    read_data_buf = pio_sm_get_blocking( pio, sm );                       /* RX_FIFOからデータを読み出す */
    read_data_buf = (read_data_buf >> 24) & 0x000000FF;                   /* RX_FIFOの32bit上位8bitにデータが入っているのでシフトして一時バッファへ */

    return read_data_buf;
}

/**
 * @brief パラレルバス用ステートマシン初期化
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param ifb PCL_IFB信号読み込み端子を指定
 * @param pio_out_base PCLアドレスバスA0に接続するGPIOの番号を指定
 * @param pio_out_num PCLアドレスバスとデータバスに使用する本数を指定
 * @param pio_side_base CS,WR,RDを割り当てるピンのベース番号を指定(PCLアドレスバスとデータバスの上にCS,RD,WRが来る)
 * @param pio_side_num CS,WR,RDに使用するピンの本数を指定(3本なので[3])
 * @param pio_in_base PCLデータバスD0に接続するGPIOの番号を指定
 */
void pcl_bus_init( PIO pio, uint sm, uint ifb, uint pio_out_base, uint pio_out_num, uint pio_side_base, uint pio_side_num, uint pio_in_base ){

    /* 指定したPIOのインストラクションメモリにプログラムをロードするプログラムの先頭アドレス(オフセット)が戻り値として返ってくる */
    uint offset = pio_add_program( pio, &parallel_bus_module_program );
    /* PIOの初期化を実施する(中身は[parallel_bus_module.pioに記載]) */
    parallel_bus_module_init( pio, sm, offset, pio_out_base, pio_out_num, pio_side_base, pio_side_num, pio_in_base);
    
    gpio_init( ifb );                         /* PCL_IFB信号読み込みピンを初期化 */
    gpio_set_dir( ifb, GPIO_IN );             /* PCL_IFB信号読み込みピンを入力に設定 */
    gpio_pull_up( ifb );                      /* PCL_IFB信号読み込みピンをプルアップする */
}

/**
 * @brief PCLレジスタ書き込み関数
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param ifb PCL_IFB信号読み込み端子を指定
 * @param axs 書き込む軸を指定
 * @param address アドレスを指定
 * @param data 書き込みデータを指定
 */
void write_reg_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address, uint32_t data ){
    
    uint32_t pcl_address_bytes[4] = {0, 0, 0, 0};                                         /* PCLアドレス用(1データ1byte) */
    uint32_t reg_data_bytes[4] = {0, 0, 0, 0};                                            /* PCLレジスタ書き込み用(1データ1byte) */
    uint32_t write_data = 0;                                                              /* PCLアクセスデータ(PCLアドレス+レジスタ書き込みデータ+CS,WR) */

    while( !gpio_get(ifb) );                                                              /* IFB端子がLOWなら無限待機 */
    pcl_bus_dir_change( pio, sm, true );                                                  /* PCLデータバスの方向を出力に設定 */

    address_byte( address, pcl_address_bytes );                                           /* アドレスデータをバイトごとに区切る */
    data_byte( data, reg_data_bytes );                                                    /* レジスタ書き込みデータをバイトごとに区切る */

    for(int i = 0; i < 4; i++){
        write_data = ( reg_data_bytes[i] << 10) | ((axs << 8) | pcl_address_bytes[i] );   /* PCLレジスタ書き込みデータ+軸選択データ+PCLアドレスデータをまとめる */
        write_tx_fifo( pio, sm, write_data );                                             /* ステートマシンのTX_FIFOに書き込み */
    }
}

/**
 * @brief PCLレジスタ読み出し関数
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param ifb PCL_IFB信号読み込み端子を指定
 * @param axs 書き込む軸を指定
 * @param address アドレスを指定
 * @return uint32_t レジスタ読み出し値
 */
uint32_t read_reg_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address ){
    
    uint32_t pcl_address_bytes[4] = {0, 0, 0, 0};                    /* PCLアドレス用(1データ1byte) */
    uint32_t read_data_buf[4] = {0, 0, 0, 0};                        /* PCLレジスタ読み出しデータの一時バッファ */
    uint32_t write_data = 0;                                         /* PCLアクセスデータ(PCLアドレス+レジスタ書き込みデータ+CS,WR) */
    uint32_t reg_read_data = 0;                                      /* PCLレジスタ読み出しデータ */

    while( !gpio_get(ifb) );                                         /* IFB端子がLOWなら無限待機 */
    pcl_bus_dir_change( pio, sm, false );                            /* PCLデータバスの方向を入力に設定 */

    address_byte( address, pcl_address_bytes );                      /* アドレスデータをバイトごとに区切る */
    
    for( int i = 0; i < 4; i++ ){
        write_data = 0x00 | ( (axs << 8) | pcl_address_bytes[i] );   /* PCLレジスタ書き込みデータ(読み出しなので全て[0])+軸選択データ+PCLアドレスデータをまとめる*/
        read_data_buf[i] = read_rx_fifo( pio, sm, write_data );      /* ステートマシンのRX_FIFOから読み出し */
    }

    reg_read_data = (read_data_buf[3] << 24) | (read_data_buf[2] << 16) | (read_data_buf[1] << 8) | read_data_buf[0];     /* PCL読み出しデータを最上位からの32bitでまとめる */
    printf( "%08X\n", reg_read_data );                                                                                    /* USBのUARTで送信する
                                                                                                                            USRTの設定は「スピード：115200」，「データ：8bit」，「パリティ：none」，「ストップビット：1bit」，「フロー制御：none」*/
    return reg_read_data;
}                                                                                                                         

/**
 * @brief PCLコマンド書き込み関数(間接書き込み方式)
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param ifb PCL_IFB信号読み込み端子を指定
 * @param com_axis コマンドで書き込む軸を指定
 * @param address アドレスを指定
 * @param com 動作コマンドを指定(PCL6046取説5.3に記載あり)
 */
void write_com_pcl( PIO pio, uint sm, uint ifb, uint32_t com_axis, uint32_t address, uint32_t com ){
    
    uint32_t address_bytes[2] = {0, 0};                                              /* PCLアドレス用(1データ1byte) */
    uint32_t command_bytes[2] = {0, 0};                                              /* PCL軸選択+コマンドデータ(それぞれ1byte) */
    uint32_t write_data = 0;                                                         /* PCLアクセスデータ(PCLアドレス+コマンド) */

    while( !gpio_get(ifb) );                                                         /* IFB端子がLOWなら無限待機 */
    pcl_bus_dir_change( pio, sm, true );                                             /* PCLデータバスの方向を出力に設定 */
    
    address_bytes[0] = (address >> 8) & 0x00FF;                                      /* COMB1_軸選択アドレス */
    address_bytes[1] = address & 0x00FF;                                             /* COMB0_コマンドアドレス */

    command_bytes[0] = com_axis;                                                     /* 軸選択ビット(X軸=0x01,Y軸=0x02,Z軸=0x04,U軸=0x08) */
    command_bytes[1] = com;                                                          /* コマンドデータ */

    for(int i = 0; i < 2; i++){
        write_data = (command_bytes[i] << 10) | ((0x00 << 8) | address_bytes[i]);    /* PCLに書き込むデータをまとめる */
        write_tx_fifo(pio, sm, write_data);                                          /* ステートマシンのTX_FIFOに書き込み */
    }
}

/**
 * @brief メインステータス読み出し関数
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param ifb PCL_IFB信号読み込み端子を指定
 * @param axs 書き込む軸を指定
 * @param address アドレスを指定
 * @return uint32_t ステータス読み出しデータ
 */
uint32_t read_msts_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address ){
    
    uint32_t address_bytes[2] = {0, 0};                              /* PCLアドレス用(1データ1byte) */
    uint32_t read_data_buf[2] = {0, 0};                              /* PCLレジスタ読み出しデータの一時バッファ */
    uint32_t write_data = 0;                                         /* PCLアクセスデータ(PCLアドレス+レジスタ書き込みデータ+CS,WR) */
    uint32_t reg_read_data = 0;                                      /* PCLレジスタ読み出しデータ */

    while( !gpio_get(ifb) );                                         /* IFB端子がLOWなら無限待機 */
    pcl_bus_dir_change( pio, sm, false );                            /* PCLデータバスの方向を入力に設定 */
    
    address_bytes[0] = (address >> 8) & 0x00FF;                      /* MSTSB0アドレス */
    address_bytes[1] = address & 0x00FF;                             /* MSTSB1アドレス */

    for( int i = 0; i < 2; i++ ){
        write_data = 0x00 | ((axs << 8) | address_bytes[i]);         /* PCLレジスタ書き込みデータ(読み出しなので全て[0])+軸選択データ+PCLアドレスデータをまとめる*/
        read_data_buf[i] = read_rx_fifo(pio, sm, write_data);        /* ステートマシンのRX_FIFOから読み出し */
    }

    reg_read_data = (read_data_buf[1] << 8) | read_data_buf[0];      /* PCL読み出しデータを最上位からの32bitでまとめる */
    printf( "%08X\n", reg_read_data );                               /* USBのUARTで送信する */
                                                                     /* USRTの設定は「スピード：115200」，「データ：8bit」，「パリティ：none」，「ストップビット：1bit」，「フロー制御：none」*/
    return reg_read_data;
}                                                                    


/**
 * @brief 汎用ポート_サブステータス読み出し関数(未チェック)
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param ifb PCL_IFB信号読み込み端子を指定
 * @param axs 書き込む軸を指定
 * @param address アドレスを指定
 */
void read_port_substat_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address ){
    
    uint32_t write_data = 0;                                         /* PCLアクセスデータ(PCLアドレス+レジスタ書き込みデータ+CS,WR) */
    uint32_t reg_read_data = 0;                                      /* PCLレジスタ読み出しデータ */

    while( !gpio_get(ifb) );                                         /* IFB端子がLOWなら無限待機 */
    pcl_bus_dir_change( pio, sm, false );                            /* PCLデータバスの方向を入力に設定 */
    
    write_data = 0x00 | ((axs << 8) | address);                      /* PCLレジスタ書き込みデータ(読み出しなので全て[0])+軸選択データ+PCLアドレスデータをまとめる*/
    reg_read_data = read_rx_fifo(pio, sm, write_data);               /* ステートマシンのRX_FIFOから読み出し */

    printf( "%08X\n", reg_read_data );                               /* USBのUARTで送信する */
}                                                                    /* USRTの設定は「スピード：115200」，「データ：8bit」，「パリティ：none」，「ストップビット：1bit」，「フロー制御：none」*/

/**
 * @brief 汎用出力ポート書き込み関数(未チェック)
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param ifb PCL_IFB信号読み込み端子を指定
 * @param axs 書き込む軸を指定
 * @param address アドレスを指定
 * @param data 書き込みデータを指定
 */
void write_port_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address, uint32_t data ){
    
    uint32_t write_data = 0;                                          /* PCLアクセスデータ(PCLアドレス+コマンド) */

    while( !gpio_get(ifb) );                                          /* IFB端子がLOWなら無限待機 */
    pcl_bus_dir_change( pio, sm, true );                              /* PCLデータバスの方向を出力に設定 */

    write_data = (data << 10) | ((axs << 8) | address);               /* PCLに書き込むデータをまとめる */
    write_tx_fifo(pio, sm, write_data);                               /* ステートマシンのTX_FIFOに書き込み */
}